package data_manage

import (
	"github.com/rdlucklib/rdluck_tools/orm"
	"time"
)

// TableEdbMapping ETA表格指标关系表
type TableEdbMapping struct {
	TableEdbMappingId int       `orm:"column(table_edb_mapping_id);pk"`
	TableInfoId       int       `description:"表格id"`
	EdbInfoId         int       `description:"指标id"`
	UniqueCode        string    `description:"唯一编码"`
	EdbDataColIndex   int       `description:"指标数据对应的列，从1开始"`
	EndDate           time.Time `description:"最后一次数据更新日期"`
	ModifyTime        time.Time `description:"修改时间"`
	CreateTime        time.Time `description:"创建时间"`
}

// TableEdbMappingItem ETA表格指标关系表
type TableEdbMappingItem struct {
	TableEdbMappingId int       ``
	TableInfoId       int       `description:"表格id"`
	EdbInfoId         int       `description:"指标id"`
	Source            int       `description:"指标来源"`
	UniqueCode        string    `description:"唯一编码"`
	EdbDataColIndex   int       `description:"指标数据对应的列，从1开始"`
	EndDate           time.Time `description:"最后一次数据更新日期"`
	EdbDataEndDate    time.Time `description:"指标数据更新结束日期"`
	ModifyTime        time.Time `description:"修改时间"`
	CreateTime        time.Time `description:"创建时间"`
}

// GetAllTableEdbMappingItemList 获取所有的 ETA表格指标关系列表数据
func GetAllTableEdbMappingItemList() (items []*TableEdbMappingItem, err error) {
	o := orm.NewOrm()
	o.Using("data")
	sql := ` select a.*,b.source,b.end_date edb_data_end_date from table_edb_mapping a 
 join edb_info b on a.edb_info_id=b.edb_info_id `
	_, err = o.Raw(sql).QueryRows(&items)
	return
}

// GetAllTableEdbMappingItemListByTableInfoId 根据表格id获取ETA表格所有的指标关系列表数据
func GetAllTableEdbMappingItemListByTableInfoId(tableInfoId int) (items []*TableEdbMappingItem, err error) {
	o := orm.NewOrm()
	o.Using("data")
	sql := ` select a.*,b.source,b.end_date edb_data_end_date from table_edb_mapping a 
 join edb_info b on a.edb_info_id=b.edb_info_id where a.table_info_id = ?`
	_, err = o.Raw(sql, tableInfoId).QueryRows(&items)
	return
}
